from django.contrib.contenttypes.models import ContentType
from django.db import IntegrityError
from django.test import TestCase

from nodetree.models import Node
from .models import Person


class TestNodeAndBaseContent(TestCase):

    def setUp(self):
        self.content = Person(first_name='Lukas', last_name='Buenger')
        self.content.save()
        self.content_type = ContentType.objects.get_for_model(self.content)

    def test_content_node_property(self):
        self.assertIsNone(self.content.node)

    def test_empty_node(self):
        node = Node()
        node.save()
        self.assertIsNone(node.content)

    def test_reverse_relation(self):

        node = Node(content_type=self.content_type, object_id=self.content.pk)
        node.save()

        # Test reference content -> node
        self.assertEqual(self.content.node, node)

    def test_unique_together(self):

        node = Node(content_type=self.content_type, object_id=self.content.pk)
        node.save()

        second_node = Node(content_type=self.content_type, object_id=self.content.pk)

        self.assertRaises(IntegrityError, second_node.save)

    def test_delete_with_content(self):

        node = Node(content_type=self.content_type, object_id=self.content.pk)
        node.save()
        node.delete()
        self.assertEqual(Person.objects.count(), 0)

    def test_delete_node_only(self):

        node = Node(content_type=self.content_type, object_id=self.content.pk)
        node.save()
        node.delete(force_delete_content=False)
        self.assertEqual(Person.objects.count(), 1)


# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from rest_framework import status
from rest_framework.exceptions import APIException

class NodeTreeError(Exception):
    pass

class AlreadyRegistered(NodeTreeError):
    pass

class NotRegistered(NodeTreeError):
    pass

class QueryParamError(APIException):

    detail = 'Invalid query parameter'
    status_code = status.HTTP_400_BAD_REQUEST

class URLParamError(APIException):

    detail = 'Invalid search URL parameter'
    status_code = status.HTTP_400_BAD_REQUEST


# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from rest_framework import six

class SimpleMap(object):
    """
    A simple ``dict`` wrapper . It mainly prevents its inner ``dict`` from overriding existing keys and lazy deletion.
    """

    def __init__(self, *args):
        self._map = dict()

    def set(self, key, value):
        """
        Sets a key/value pair. If the key already exist, a ``KeyError`` gets thrown.

        :param key:
        :type key: \*
        :param value:
        :type value: \*
        :raises: KeyError

        """
        if key in self._map:
            raise KeyError('Key "%s" is already taken. "SimpleMap" does not allow to override keys.' % key)
        self._map[key] = value

    def remove(self, key):
        """
        Removes a key/value pair based upon the given *key*. If the key doesn't exist, a ``KeyError`` gets thrown.

        :param key:
        :type key: \*

        :raises: KeyError

        """
        if not key in self._map:
            raise KeyError('Key "%s" does not exist. "SimpleMap" does not allow to lazily delete non-existing keys.' % key)
        del self._map[key]

    def has(self, key):
        """
        Returns whether a key exists.

        :param key:
        :type key: \*

        :rtype: bool
        """

        return key in self._map

    def get(self, key):
        """
        Returns the value for *key* if present, else ``None``.

        :param key:
        :type key: \*

        :rtype: \* or None
        """
        return self._map.get(key, None)

    def values(self, *keys):
        """
        Returns a list of values for the related *\*keys*. If *\*keys* is empty, all values get returned.

        :param \*keys:
        :type \*keys: list of keys

        :rtype: list
        """

        if len(keys) < 1:
            return six.itervalues(self._map)
        return [value for key, value in six.iteritems(self._map) if key in keys]

    def keys(self):
        """
        Returns a list of all keys.

        :rtype: list
        """
        return six.iterkeys(self._map)


    def length(self):
        """
        Returns the number of items in this map.

        :rtype: int
        """
        return len(self._map)


# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Node'
        db.create_table(u'nodetree_node', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('content_type', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['contenttypes.ContentType'], null=True)),
            ('object_id', self.gf('django.db.models.fields.PositiveIntegerField')(null=True)),
            ('parent', self.gf('mptt.fields.TreeForeignKey')(blank=True, related_name=u'children', null=True, to=orm['nodetree.Node'])),
            ('node_type', self.gf('django.db.models.fields.SlugField')(max_length=31)),
            ('lft', self.gf('django.db.models.fields.PositiveIntegerField')(db_index=True)),
            ('rght', self.gf('django.db.models.fields.PositiveIntegerField')(db_index=True)),
            ('tree_id', self.gf('django.db.models.fields.PositiveIntegerField')(db_index=True)),
            ('level', self.gf('django.db.models.fields.PositiveIntegerField')(db_index=True)),
        ))
        db.send_create_signal(u'nodetree', ['Node'])

        # Adding unique constraint on 'Node', fields ['content_type', 'object_id']
        db.create_unique(u'nodetree_node', ['content_type_id', 'object_id'])


    def backwards(self, orm):
        # Removing unique constraint on 'Node', fields ['content_type', 'object_id']
        db.delete_unique(u'nodetree_node', ['content_type_id', 'object_id'])

        # Deleting model 'Node'
        db.delete_table(u'nodetree_node')


    models = {
        u'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        u'nodetree.node': {
            'Meta': {'unique_together': "((u'content_type', u'object_id'),)", 'object_name': 'Node'},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']", 'null': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'level': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'lft': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'node_type': ('django.db.models.fields.SlugField', [], {'max_length': '31'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True'}),
            'parent': ('mptt.fields.TreeForeignKey', [], {'blank': 'True', 'related_name': "u'children'", 'null': 'True', 'to': u"orm['nodetree.Node']"}),
            'rght': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'tree_id': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'})
        }
    }

    complete_apps = ['nodetree']


from __future__ import unicode_literals

from django.test import TestCase

import nodetree
from nodetree.exceptions import AlreadyRegistered, NotRegistered

from .nodetypes import PersonNodeType
from .models import Town, Person
from nodetree.settings import nodetree_settings


class TestNodeTree(TestCase):

    def test_register(self):
        self.assertRaises(AlreadyRegistered, nodetree.app.register, PersonNodeType)

    def test_unregister(self):
        nodetree.app.unregister(PersonNodeType)
        self.assertFalse(nodetree.app.has_node_type('person'))

        with self.assertRaises(NotRegistered):
            nodetree.app.unregister(PersonNodeType)

        nodetree.app.register(PersonNodeType)

    def test_get_node_type(self):
        self.assertIsNone(nodetree.app.get_node_type('foo'))
        self.assertEqual(nodetree.app.get_node_type('town'), nodetree.app._map.get('town'))

    def test_has_node_type(self):
        self.assertFalse(nodetree.app.has_node_type('foo'))
        self.assertTrue(nodetree.app.has_node_type('area'))

    def test_get_node_type_for_model(self):
        node_type_obj = nodetree.app.get_node_type('person')
        self.assertEqual(node_type_obj, nodetree.app.get_node_type_for_model(Person))

        person = Person(first_name='Lukas', last_name='Buenger')
        self.assertEqual(node_type_obj, nodetree.app.get_node_type_for_model(person))

    def test_get_generic_field_dict(self):
        generic_field_dict = nodetree.app.get_generic_field_dict()
        self.assertIn(Town, generic_field_dict)
        self.assertIsInstance(generic_field_dict[Town], nodetree_settings.DEFAULT_CONTENT_SERIALIZER)

    def test_get_node_serializer_class(self):
        serializer = nodetree.app.get_node_serializer_class()()
        self.assertIn(Town, serializer.fields['content'].serializers)
        self.assertIsInstance(serializer, nodetree_settings.BASE_NODE_SERIALIZER)

    def test_get_update_node_serializer_class(self):
        serializer = nodetree.app.get_update_node_serializer_class()()
        self.assertIsInstance(serializer, nodetree_settings.BASE_UPDATE_NODE_SERIALIZER)

    def test_get_create_node_serializer_class(self):
        serializer = nodetree.app.get_create_node_serializer_class()()
        self.assertIn(Town, serializer.fields['content'].serializers)
        self.assertIsInstance(serializer, nodetree_settings.BASE_CREATE_NODE_SERIALIZER)


# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import re

from django.db import models
from django.core.exceptions import ImproperlyConfigured

from rest_framework.serializers import Serializer

from nodetree.settings import nodetree_settings


NAME_PATTERN = re.compile(r'[\-]*\w{1,31}')

#TODO: Write proper representation functions for all items

class NodeType(object):
    """
    With subclasses of ``NodeType`` you define and hook up all parts that are needed to establish a NodeTree entity.
    It's also the object you'll register to the ``NodeTree`` app.
    """

    name = None
    """
    A unique name for this ``NodeType``. Can only contain small characters and minus symbols (``r'[\-]*\w')``. This
    property has to be set for each subclass, otherwise ``ImproperlyConfigured`` is raised.

    .. note::
       This will be the main identifier for the associated ``model_class`` within the whole NodeTree world.
    """

    model_class = None
    """
    The content model you want to use with this ``NodeType``. Subclass of ``BaseContent``. This property has to be
    set for each subclass, otherwise ``ImproperlyConfigured`` is raised.
    """

    serializer_class = None
    """
    A ``Serializer`` class or subclass that should get used to expose the ``model_class`` to the NodeTree API.
    By default ``nodetree_settings.DEFAULT_CONTENT_SERIALIZER`` is used.
    """

    view_set_class = None

    allowed_children = []
    """
    A list of ``NodeType`` names that are allowed to be added as children to a node of this type. If the list is empty,
    the ``NodeType`` behaves as if the ``is_leaf`` property is set to ``True``.
    """

    forbidden_children = []

    is_root = False
    """
    If ``True`` this ``NodeType`` can be created without a parent node or moved to a root position.
    """

    is_leaf = False
    """
    Adding child nodes to objects of this ``NodeType`` will not be allowed. ``allowed_children`` will be ignored
    if ``is_leaf`` is ``True``.
    """

    def __init__(self):
        """
         :raises: django.core.exceptionsImproperlyConfigured
        """

        if self.name is None:
            raise ImproperlyConfigured("'%r' has to have its 'name' property set." % self.__class__)
        elif not NAME_PATTERN.match(self.name):
            raise ImproperlyConfigured("'%r' has an invalid 'name' property." % self.__class__)

        if self.model_class is None:
            raise ImproperlyConfigured("'%r' has to have its 'model_class' property set." % self.__class__)
        elif not issubclass(self.model_class, models.Model):
            raise ImproperlyConfigured("'%r' has an invalid 'model_class'." % self.__class__)

        if self.serializer_class is None:
            class DefaultSerializer(nodetree_settings.DEFAULT_CONTENT_SERIALIZER):
                class Meta:
                    model = self.model_class
            self.serializer_class = DefaultSerializer
        elif not issubclass(self.serializer_class, Serializer):
            raise ImproperlyConfigured("'%r' has an invalid 'serializer_class'." % self.__class__)

    def is_allowed_child(self, name):
        return self.is_leaf is not True and \
               (len(self.forbidden_children) > 0 and name not in self.forbidden_children or \
               nodetree_settings.ANY_KEYWORD in self.allowed_children or \
               name in self.allowed_children)

    @property
    def view_set(self):
        view_set_class = self.view_set_class or nodetree_settings.DEFAULT_CONTENT_VIEW_SET
        class ContentViewSet(view_set_class):
            serializer_class = self.serializer_class or nodetree_settings.DEFAULT_CONTENT_SERIALIZER
            queryset = self.model_class.objects.all()
        return ContentViewSet


    def __repr__(self):
        return '<NodeType: %s>' % self.name


# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db.models import Model

from rest_framework.relations import HyperlinkedRelatedField
from rest_framework.routers import DefaultRouter

from nodetree.fields import ContentField, RecursiveRelatedField
from nodetree.maps import SimpleMap
from nodetree.exceptions import AlreadyRegistered, NotRegistered
from nodetree.serializers import BaseNodeSerializer, BaseUpdateNodeSerializer, BaseCreateNodeSerializer, BaseInputNodeSerializer
from nodetree.settings import nodetree_settings
from nodetree.viewsets import NodeViewSet

class NodeTree(object):
    """
    The main NodeTree application that exposes an interface to register, create and retrieve ``NodeType`` objects. It
    acts as registration pool towards your app and as facade for the other members of the NodeTree world e.g. the
    validation procedure of ``BaseUpdateNodeSerializer``.

    It gets instantiated within the same package and as for now behaves as a singleton,
    similar to e.g. the ``django.admin.site`` object. In fact, its interface tries to resemble that of
    ``django.admin.site``.

    .. note::
       If you intend to subclass or extend NodeTree components, please keep in mind, that you should always use
       ``nodetree.app`` to establish communication between different parts of the application (resources, models,
       ``NodeTypes`` and so on) to not break the `Facade pattern <http://en.wikipedia.org/wiki/Facade_pattern>`_.
    """

    def __init__(self):
        """
        Creates an instance of ``nodetree.base.SimpleMap`` to register ``NodeType``s
        """
        self._map = SimpleMap()

    def register(self, node_type):
        """
        With this method you make a ``NodeType`` available for NodeTree. Raises ``nodetree.exceptions.AlreadyRegistered``
        if the given ``NodeType`` is already registered.

        :param node_type: A subclass of ``NodeType``.
        :type node_type: nodetree.nodetypes.NodeType

        :raises: nodetree.exceptions.AlreadyRegistered
        """
        try:
            node_type = node_type()
            self._map.set(node_type.name, node_type)
        except KeyError:
            raise AlreadyRegistered('"%r" is already registered to NodeTree' % node_type)

    def unregister(self, node_type):
        try:
            node_type = node_type()
            self._map.remove(node_type.name)
        except KeyError:
            raise NotRegistered('"%r" is not registered to NodeTree' % node_type)

    def get_node_type(self, node_type_name):
        """
        A shortcut to the ``get`` method of the internal ``nodetree.base.SimpleMap`` object.

        :param node_type_name: A valid name of a ``NodeType`` that has been registered to NodeTree.
        :type node_type_name: str or unicode

        :rtype: nodetree.NodeType

        .. note:
           As for now, this method fails silently if a key doesn't exist. However, this behavior may change in favor of
           a stricter error handling
        """
        return self._map.get(node_type_name)

    def get_node_type_for_model(self, model):

        model_class = None
        if isinstance(model, type):
            model_class = model
        elif isinstance(model, Model):
            model_class = model.__class__

        if model_class is not None:
            for node_type_obj in self._map.values():
                if node_type_obj.model_class == model_class:
                    return node_type_obj
        return None

    def has_node_type(self, node_type_name):
        """
        :param node_type_name: A valid name of a ``NodeType`` that you want to check against.
        :type node_type_name: str or unicode

        :rtype: bool
        """
        return node_type_name in self._map.keys()

    def get_generic_field_dict(self):
        """
        Creates and returns the dictionary for the generated ``nodetree.fields.ContentField``.

        .. note:
           As for now the implementation of ``GenericRelatedField`` is still
           `in revision <https://github.com/tomchristie/django-rest-framework/pull/755>`_.

        :rtype: dict
        """
        generic_field_dict = dict()

        for node_type in self._map.values():
            generic_field_dict[node_type.model_class] = node_type.serializer_class()

        return generic_field_dict

    def get_node_serializer_class(self, depth=nodetree_settings.DEFAULT_MAX_DEPTH):
        """
        Creates and returns a subclass of ``BaseNodeSerializer`` with a ``ContentField`` that has a mapping for
        all models registered to NodeTree.

        :rtype: nodetree.serializer.BaseNodeSerializer
        """

        class NodeSerializer(BaseNodeSerializer):
            app = self
            content = ContentField(self.get_generic_field_dict(), read_only=True)
            children = RecursiveRelatedField(many=True, max_depth=depth,
                leaf_serializer=HyperlinkedRelatedField(view_name='node-detail', many=True))
        return NodeSerializer

    def get_update_node_serializer_class(self, depth=nodetree_settings.DEFAULT_MAX_DEPTH):
        """
        Returns ``BaseUpdateNodeSerializer``.

        :rtype: nodetree.serializer.BaseUpdateNodeSerializer
        """
        class UpdateNodeSerializer(BaseUpdateNodeSerializer):
            max_depth = depth
        return UpdateNodeSerializer

    def get_create_node_serializer_class(self, depth=nodetree_settings.DEFAULT_MAX_DEPTH):
        """
        Creates and returns a subclass of ``BaseCreateNodeSerializer`` with a ``ContentField`` that has a mapping for
        all models registered to NodeTree.

        :rtype: nodetree.serializer.BaseCreateNodeSerializer
        """
        class CreateNodeSerializer(BaseCreateNodeSerializer):
            max_depth = depth
            content = ContentField(self.get_generic_field_dict(), required=True)
        return CreateNodeSerializer


    @property
    def urls(self):
        router = DefaultRouter()
        router.register(r'node', NodeViewSet)
        for node_type in self._map.values():
            router.register(r'%s' % node_type.name, node_type.view_set)
        return router.urls


app = NodeTree()


# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import json

from django.contrib.auth.models import Permission, User
from django.test import TestCase, Client

from rest_framework.compat import patterns, include, url

import nodetree
from nodetree.models import Node
from nodetree.permissions import NodeTreeModelPermissions
from nodetree.viewsets import NodeViewSet

from .models import Town, Area, House

nodetree.autodiscover()
urlpatterns = patterns('',
    url(r'^nodetree/node/$', NodeViewSet.as_view(
            {'get': 'list', 'post': 'create'},
            permission_classes = (NodeTreeModelPermissions, )
        ), name='node-list'),
    url(r'^nodetree/node/(?P<pk>[0-9]+)/$', NodeViewSet.as_view(
            actions={'get': 'retrieve', 'put': 'update', 'delete': 'destroy'},
            permission_classes = (NodeTreeModelPermissions, )
        ), name='node-detail'),
    url(r'^nodetree/', include(nodetree.app.urls)),
)

client = Client()

class TestNodeTreeModelPermissions(TestCase):

    urls = 'nodetree.tests.permissions'

    def setUp(self):
        self.town = Town.objects.create(name='Zurich')
        self.town_node = Node.objects.create(node_type='town', content=self.town)

        self.area = Area.objects.create(name='Wiedikon')
        self.area_node = Node.objects.create(node_type='area', content=self.area, parent=self.town_node)

        self.house = House.objects.create(name='Kalkbreitestrasse 98')
        self.house_node = Node.objects.create(node_type='house', content=self.house, parent=self.area_node)

        self.user = User.objects.create_user('lukasbuenger', 'lukasbuenger@gmail.com', password='any-password')
        for code_name in ['add_node', 'add_town', 'change_node', 'change_town', 'add_person', 'change_person']:
            permission = Permission.objects.get(codename=code_name)
            self.user.user_permissions.add(permission)

        client.login(username='lukasbuenger', password='any-password')

    def test_post_list_permissions(self):
        response = client.post('/nodetree/node/', json.dumps({
            'node_type': 'town',
            'content': {
                'name': 'Berne',
            }
        }), content_type='application/json')
        self.assertEqual(response.status_code, 201)

    def test_not_provided_post_list_permissions(self):
        response = client.post('/nodetree/node/', json.dumps({
            'node_type': 'area',
            'content': {
                'name': 'Seefeld',
            }
        }), content_type='application/json')
        self.assertEqual(response.status_code, 403)

    def test_get_list_permissions(self):
        response = client.get('/nodetree/node/')
        self.assertEqual(response.status_code, 200)

    def test_invalid_put_detail_permission(self):
        response = client.put('/nodetree/node/3/', json.dumps({
            'target': '/nodetree/node/1/',
            'position': 'first-child',
        }), content_type='application/json')
        self.assertEqual(response.status_code, 403)


# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from nodetree.apps import app
from nodetree.nodetypes import NodeType


def autodiscover():
    """
    A simple clone of the ``django.contrib.admin.autodiscover`` method.

    The original docstring with adjusted NodeTree names:
    Auto-discover INSTALLED_APPS nodetypes.py modules and fail silently when
    not present. This forces an import on them to register any NodeTree bits they
    may want.
    """
    import copy
    from django.conf import settings
    from django.utils.importlib import import_module
    from django.utils.module_loading import module_has_submodule

    for module in settings.INSTALLED_APPS:
        mod = import_module(module)
        # Attempt to import the module's nodetypes module.
        try:
            before_import_map = copy.copy(app._map._map)
            import_module('%s.nodetypes' % module)
        except:
            app._map._map = before_import_map
            if module_has_submodule(mod, 'nodetypes'):
                raise


# -*- coding: utf-8 -*- 
from __future__ import unicode_literals

import six

from rest_framework.filters import BaseFilterBackend

from nodetree.exceptions import QueryParamError

try:
    import microfilters
except ImportError:
    microfilters = None


FilterSet = microfilters and microfilters.FilterSet or None
Filter = microfilters and microfilters.Filter or None

if microfilters:
    class NodeTypeFilter(Filter):

        def sanitize(self, value):
            if not isinstance(value, six.text_type):
                raise QueryParamError
            node_type_list = value.split(',')
            return list(filter(lambda x: value != '', node_type_list))

        def filter(self, queryset, value):
            return queryset.filter(node_type__in=value)


    class LevelFilter(Filter):

        def sanitize(self, value):
            try:
                value = int(value)
            except Exception:
                value = -1
            if value < 0:
                raise QueryParamError
            return value


    class FromLevelFilter(LevelFilter):

        def filter(self, queryset, value):
            return queryset.filter(level__gte=value)


    class ToLevelFilter(LevelFilter):

        def filter(self, queryset, value):
            return queryset.filter(level__lte=value)


    class NodeFilterSet(FilterSet):

        def sanitize(self, query_dict):
            return dict((k, v) for k, v in six.iteritems(query_dict) if (v == n for n in six.iterkeys(self._filters)))

        def __init__(self):
            super(NodeFilterSet, self).__init__({
                'node_type': NodeTypeFilter(),
                'from_level': FromLevelFilter(),
                'to_level': ToLevelFilter()
            })


class MicrofiltersFilterBackend(BaseFilterBackend):

    """
    A filter backend that uses microfilters.
    """

    def __init__(self):
        assert microfilters, 'Using MicrofiltersFilterBackend, but microfilters is not installed'

    def get_filter_set(self, view):

        filter_set = getattr(view, 'filter_set', None)
        return filter_set

    def filter_queryset(self, request, queryset, view):
        filter_set = self.get_filter_set(view)

        if filter_set:
            query_dict = filter_set.sanitize(request.QUERY_PARAMS)
            return filter_set.filter(queryset, query_dict)
        return queryset


if microfilters:
    node_filter_set = NodeFilterSet()
    node_filter_backend = MicrofiltersFilterBackend
else:
    node_filter_backend = None
    node_filter_set = None


# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.core.exceptions import ValidationError
from django.utils.translation import ugettext_lazy as _

from rest_framework import serializers

import nodetree
from nodetree.models import Node
from nodetree.settings import nodetree_settings


class BaseContentSerializer(serializers.HyperlinkedModelSerializer):

    node = serializers.HyperlinkedRelatedField(view_name='node-detail', read_only=True)

    class Meta:
        exclude = ('id', )


class BaseNodeSerializer(serializers.HyperlinkedModelSerializer):

    parent = serializers.HyperlinkedRelatedField(view_name='node-detail')

    class Meta:
        model = Node
        exclude = ('content_type', 'object_id', 'id', 'lft', 'rght', 'tree_id', )


class BaseInputNodeSerializer(serializers.Serializer):

    max_depth = None

    target = serializers.HyperlinkedRelatedField(view_name='node-detail', queryset=Node.objects.all(), required=False)
    position = serializers.ChoiceField(choices=(
            ('first-child', _('First child')),
            ('last-child', _('Last child')),
            ('left', _('Left')),
            ('right', _('Right')),
        ), required=False)

    nodetree_error_messages = {
        'invalid_root': _("Node type '%s' can't be used as root." ),
        'position_required': _("You have to provide a 'position' in combination with a 'target'." ),
        'invalid_child': _("You can't add node type '%s' as child of '%s'." ),
        'invalid_node_type': _("Node type '%s' does not exist." ),
        'invalid_target': _("You can't use the object you want to move as target." ),
    }

    def __init__(self, *args, **kwargs):
        if self.max_depth is None:
            self.max_depth = nodetree_settings.DEFAULT_MAX_DEPTH
        super(BaseInputNodeSerializer, self).__init__(error_messages=self.nodetree_error_messages, *args, **kwargs)

    def get_parent(self, target, position):

        if position in ('left', 'right'):
            return target.parent
        else:
            return target

    def get_node_type(self, node_type):
        return nodetree.app.get_node_type(node_type)

    def has_node_type(self, node_type):
        return nodetree.app.has_node_type(node_type)

    def get_output_serializer(self):
        return nodetree.app.get_node_serializer_class(depth=self.max_depth)

    def validate(self, attrs):
        node_type = attrs.get('node_type')
        target = attrs.get('target', None)
        position = attrs.get('position', None)
        node_type_obj = self.get_node_type(node_type)

        if target is None:
            if node_type_obj.is_root is not True:
                raise ValidationError(self.error_messages['invalid_root'] % node_type_obj.name)
            return attrs
        else:
            if position is None:
                raise ValidationError(self.error_messages['position_required'])
            parent = self.get_parent(target, position)
            if parent is None and node_type_obj.is_root is not True:
                raise ValidationError(self.error_messages['invalid_root'] % node_type_obj.name)
            parent_node_type_obj = self.get_node_type(parent.node_type)
            if not parent_node_type_obj.is_allowed_child(node_type_obj.name):
                raise ValidationError(self.error_messages['invalid_child'] % (node_type_obj.name, parent_node_type_obj.name))

        return attrs

    def restore_object(self, attrs, instance=None):
        if instance is not None:
            Node.objects.move_node(instance, attrs.get('target'), attrs.get('position'))
        else:
            node_type_obj = self.get_node_type(attrs.get('node_type'))
            content = attrs.get('content')
            content.save()
            instance = Node(node_type=node_type_obj.name, content=content)
            Node.objects.insert_node(instance, attrs.get('target'), attrs.get('position'))
        return instance

    @property
    def data(self):
        if not self.object:
            raise Exception("You can't call the 'data' attribute of a 'BaseNodeInputSerializer' that doesn't represent "
                            "an existing 'Node' object.")
        return self.get_output_serializer()(self.object, context=self.context).data


class BaseUpdateNodeSerializer(BaseInputNodeSerializer):

    def validate(self, attrs):
        if attrs['target'] == self.object:
            raise ValidationError(self.error_messages['invalid_target'])
        attrs['node_type'] = self.object.node_type
        attrs = super(BaseUpdateNodeSerializer, self).validate(attrs)
        parent = self.get_parent(attrs['target'], attrs['position'])
        if parent is not None and parent == self.object:
            raise ValidationError(self.error_messages['invalid_target'])
        return attrs


class BaseCreateNodeSerializer(BaseInputNodeSerializer):

    node_type = serializers.SlugField(max_length=31)

    def validate_node_type(self, attrs, source):

        node_type = attrs[source]
        if not self.has_node_type(node_type):
            raise ValidationError(self.error_messages['invalid_node_type'] % node_type)
        return attrs


from __future__ import unicode_literals

from django.test import TestCase

from nodetree.maps import SimpleMap


class TestSimpleMap(TestCase):

    def setUp(self):
        self.map = SimpleMap()

    def test_instantiation(self):
        self.assertIsInstance(self.map._map, dict)

    def test_set_method(self):
        self.map.set('One', 1)

        self.assertEqual(len(self.map._map), 1)
        self.assertTrue(self.map.has('One'))
        self.assertFalse(self.map.has('Two'))
        self.assertRaises(KeyError, self.map.set, 'One', 1)

        self.map.set('Two', 2)

        self.assertEqual(len(self.map._map), 2)
        self.assertTrue(self.map.has('One'))
        self.assertTrue(self.map.has('Two'))
        self.assertRaises(KeyError, self.map.set, 'Two', 1)

    def test_remove_method(self):
        self.map.set('One', 1)
        self.map.set('Two', 2)

        self.map.remove('One')

        self.assertEqual(len(self.map._map), 1)
        self.assertEqual(self.map.length(), 1)
        self.assertFalse(self.map.has('One'))
        self.assertTrue(self.map.has('Two'))
        self.assertRaises(KeyError, self.map.remove, 'One')

        self.map.remove('Two')

        self.assertEqual(len(self.map._map), 0)
        self.assertEqual(self.map.length(), 0)
        self.assertFalse(self.map.has('One'))
        self.assertFalse(self.map.has('Two'))
        self.assertRaises(KeyError, self.map.remove, 'Two')

    def test_has_method(self):
        self.map.set('One', 1)

        self.assertTrue(self.map.has('One'))
        self.assertFalse(self.map.has('Two'))

    def test_get_method(self):
        self.map.set('One', 1)
        self.map.set('Two', 2)

        self.assertEqual(self.map.get('One'), 1)
        self.assertEqual(self.map.get('Two'), 2)
        self.assertIsNone(self.map.get('Three'))

    def test_values_method(self):
        self.map.set('One', 1)
        self.map.set('Two', 2)

        self.assertTrue(1 in self.map.values() and 2 in self.map.values())
        self.assertTrue(1 in self.map.values('One') and not 2 in self.map.values('One'))
        self.assertTrue(not 1 in self.map.values('Two') and 2 in self.map.values('Two'))
        self.assertTrue(1 in self.map.values('One', 'Two') and 2 in self.map.values('One', 'Two'))


    def test_keys_method(self):
        self.map.set('One', 1)
        self.map.set('Two', 2)

        self.assertIn('One', self.map.keys())
        self.assertIn('Two', self.map.keys())

        self.map.remove('One')
        self.assertIn('Two', self.map.keys())
        self.assertFalse('One' in self.map.keys())





# -*- coding: utf-8 -*- 
from __future__ import unicode_literals

from rest_framework.permissions import DjangoModelPermissions

import nodetree


class NodePermissionMixin(object):

    def get_model_class(self, request, view):
        try:
            node_type = view.get_object().node_type
        except:
            node_type = request.DATA.get('node_type')
        if node_type is not None:
            node_type_obj = nodetree.app.get_node_type(node_type)
            if node_type_obj:
                return node_type_obj.model_class
        return None


class NodeTreeModelPermissions(NodePermissionMixin, DjangoModelPermissions):

    def has_permission(self, request, view):

        if not super(NodeTreeModelPermissions, self).has_permission(request, view):
            return False

        model_cls = self.get_model_class(request, view)
        if not model_cls:
            return True

        perms = self.get_required_permissions(request.method, model_cls)


        if (request.user and
            (request.user.is_authenticated() or not self.authenticated_users_only) and
            request.user.has_perms(perms)):
            return True
        return False


# -*- coding: utf-8 -*- 
from __future__ import unicode_literals

from django.core.exceptions import ObjectDoesNotExist
from django.http import Http404

from rest_framework.decorators import link
from rest_framework.generics import GenericAPIView
from rest_framework.mixins import RetrieveModelMixin, UpdateModelMixin, ListModelMixin
from rest_framework.permissions import SAFE_METHODS
from rest_framework.settings import api_settings
from rest_framework.viewsets import ModelViewSet, ViewSetMixin

import nodetree
from nodetree.exceptions import QueryParamError
from nodetree.filters import node_filter_set, node_filter_backend
from nodetree.models import Node
from nodetree.settings import nodetree_settings


class BaseContentViewSet(RetrieveModelMixin, UpdateModelMixin, ListModelMixin, ViewSetMixin, GenericAPIView):
    http_method_names = ['get', 'put', 'head', 'options']


class NodeViewSet(ModelViewSet):


    http_method_names = ['get', 'post', 'put', 'delete', 'head', 'options']
    permission_classes = nodetree_settings.NODE_PERMISSIONS or api_settings.DEFAULT_PERMISSION_CLASSES
    model = Node
    filter_set = node_filter_set
    filter_backend = node_filter_backend

    def get_depth(self):
        try:
            depth = self.request.QUERY_PARAMS.get('depth', nodetree_settings.DEFAULT_MAX_DEPTH)
            return int(depth)
        except ValueError:
            raise QueryParamError

    def get_serializer_class(self):
        if self.request.method in SAFE_METHODS or self.request.method == 'DELETE':
            return nodetree.app.get_node_serializer_class(self.get_depth())
        elif self.request.method == 'POST':
            return nodetree.app.get_create_node_serializer_class(self.get_depth())
        elif self.request.method in ['PUT', 'PATCH']:
            return nodetree.app.get_update_node_serializer_class(self.get_depth())


    def get_node(self, pk):
        try:
            node = Node.objects.get(pk=pk)
        except ObjectDoesNotExist:
            raise Http404
        return node

    @link()
    def ancestors(self, request, pk=None):
        self.queryset = self.get_node(pk).get_ancestors()
        return self.list(request)

    @link()
    def descendants(self, request, pk=None):
        self.queryset = self.get_node(pk).get_descendants()
        return self.list(request)

    @link()
    def children(self, request, pk=None):
        self.queryset = self.get_node(pk).get_children()
        return self.list(request)

    @link()
    def siblings(self, request, pk=None):
        self.queryset = self.get_node(pk).get_siblings()
        return self.list(request)

    @link()
    def root(self, request, pk=None):
        self.kwargs['pk'] = self.get_node(pk).get_root().pk
        return self.retrieve(request)



from __future__ import unicode_literals

from django.test import TestCase

from nodetree.settings import nodetree_settings
from nodetree.serializers import BaseContentSerializer, BaseNodeSerializer, BaseUpdateNodeSerializer, BaseCreateNodeSerializer

# TODO: Test exception handling
from nodetree.viewsets import BaseContentViewSet, NodeViewSet

class TestNodeTreeSettingsDefault(TestCase):

    def test_default_content_serializer(self):
        self.assertEqual(nodetree_settings.DEFAULT_CONTENT_SERIALIZER, BaseContentSerializer)

    def test_default_content_view_set(self):
        self.assertEqual(nodetree_settings.DEFAULT_CONTENT_VIEW_SET, BaseContentViewSet)

    def test_node_view_set(self):
        self.assertEqual(nodetree_settings.NODE_VIEW_SET, NodeViewSet)

    def test_default_max_depth(self):
        self.assertEqual(nodetree_settings.DEFAULT_MAX_DEPTH, 0)

    def test_base_node_serializer(self):
        self.assertEqual(nodetree_settings.BASE_NODE_SERIALIZER, BaseNodeSerializer)

    def test_base_update_node_serializer(self):
        self.assertEqual(nodetree_settings.BASE_UPDATE_NODE_SERIALIZER, BaseUpdateNodeSerializer)

    def test_base_create_node_serializer(self):
        self.assertEqual(nodetree_settings.BASE_CREATE_NODE_SERIALIZER, BaseCreateNodeSerializer)

    def test_node_permissions(self):
        self.assertEqual(nodetree_settings.NODE_PERMISSIONS, None)


from __future__ import unicode_literals

from django.core.exceptions import ImproperlyConfigured
from django.test import TestCase

from rest_framework import serializers

from nodetree.nodetypes import NodeType
from nodetree.serializers import BaseContentSerializer

from .models import Town


class TownSerializer(serializers.ModelSerializer):

    class Meta:
        model = Town


class TestNodeType(TestCase):

    def test_non_existing_name(self):

        class TownNodeType(NodeType):
            pass

        with self.assertRaises(ImproperlyConfigured):
            node_type = TownNodeType()

    def test_invalid_name(self):

        class TownNodeType(NodeType):
            name = 'foooooooooooooooooo-baaaaaaaaaaaaaaar'

        with self.assertRaises(ImproperlyConfigured):
            node_type = TownNodeType()

        class AnotherTownNodeType(NodeType):
                    name = 'FooBar'

        with self.assertRaises(ImproperlyConfigured):
            node_type = AnotherTownNodeType()

    def test_non_existing_model(self):

        class TownNodeType(NodeType):
            name = 'foo-bar'

        with self.assertRaises(ImproperlyConfigured):
            node_type = TownNodeType()

    def test_invalid_model(self):

        class TownNodeType(NodeType):
            name = 'foo-bar'
            model_class = dict

        with self.assertRaises(ImproperlyConfigured):
            node_type = TownNodeType()

    def test_default_serializer(self):

        class TownNodeType(NodeType):
            name = 'foo-bar'
            model_class = Town

        node_type = TownNodeType()
        self.assertTrue(issubclass(node_type.serializer_class, BaseContentSerializer))
        self.assertEqual(node_type.serializer_class.Meta.model, Town)


    def test_any_string(self):

        class TownNodeType(NodeType):
            name = 'foo-bar'
            model_class = Town
            allowed_children = ['any']

        node_type = TownNodeType()
        self.assertTrue(node_type.is_allowed_child('blub'))

    def test_not_any_string(self):

        class TownNodeType(NodeType):
            name = 'foo-bar'
            model_class = Town
            allowed_children = ['foo']

        node_type = TownNodeType()
        self.assertFalse(node_type.is_allowed_child('blub'))

    def test_invalid_custom_serializer(self):

        class TownNodeType(NodeType):
            name = 'foo-bar'
            model_class = Town
            serializer_class = dict

        with self.assertRaises(ImproperlyConfigured):
            node_type = TownNodeType()

    def test_custom_serializer(self):

        class TownNodeType(NodeType):
            name = 'foo-bar'
            model_class = Town
            serializer_class = TownSerializer

        node_type = TownNodeType()
        self.assertEqual(node_type.serializer_class, TownSerializer)


"""
Blank URLConf just to keep runtests.py happy.
"""
from rest_framework.compat import patterns, url, include
import nodetree

urlpatterns = patterns('',
#    url(r'^nodetree/', include(nodetree.app.urls)),
)



# -*- coding: utf-8 -*-
# Inspired by Django REST Framework's excellent settings handling
from __future__ import unicode_literals
from rest_framework import six

from django.conf import settings
from django.utils import importlib

USER_SETTINGS = getattr(settings, 'NODETREE', None)

DEFAULTS = {
    'DEFAULT_CONTENT_SERIALIZER': 'nodetree.serializers.BaseContentSerializer',
    'DEFAULT_CONTENT_VIEW_SET': 'nodetree.viewsets.BaseContentViewSet',
    'NODE_VIEW_SET': 'nodetree.viewsets.NodeViewSet',
    'NODE_PERMISSIONS': None,
    'BASE_NODE_SERIALIZER': 'nodetree.serializers.BaseNodeSerializer',
    'BASE_UPDATE_NODE_SERIALIZER': 'nodetree.serializers.BaseUpdateNodeSerializer',
    'BASE_CREATE_NODE_SERIALIZER': 'nodetree.serializers.BaseCreateNodeSerializer',
    'DEFAULT_MAX_DEPTH': 0,
    'ANY_KEYWORD': 'any',
}

# List of settings that may be in string import notation.
IMPORT_STRINGS = (
    'BASE_NODE_SERIALIZER',
    'BASE_UPDATE_NODE_SERIALIZER',
    'BASE_CREATE_NODE_SERIALIZER',
    'DEFAULT_CONTENT_SERIALIZER',
    'DEFAULT_CONTENT_VIEW_SET',
    'NODE_VIEW_SET',
    'NODE_PERMISSIONS',
)

def perform_import(val, setting_name):
    """
    If the given setting is a string import notation,
    then perform the necessary import or imports.

    :param val: Value for the given ``setting_name`
    :type val: list or str or unicode

    :param setting_name: Name of the setting parameter
    :type setting_name: str or unicode

    :rtype: *
    """
    if isinstance(val, six.string_types):
        return import_from_string(val, setting_name)
    elif isinstance(val, (list, tuple)):
        return [import_from_string(item, setting_name) for item in val]
    return val


def import_from_string(val, setting_name):
    """
    Attempt to import a class from a string representation.
    """
    try:
        # Nod to tastypie's use of importlib.
        parts = val.split('.')
        module_path, class_name = '.'.join(parts[:-1]), parts[-1]
        module = importlib.import_module(module_path)
        return getattr(module, class_name)
    except ImportError as e:
        msg = "Could not import '%s' for API setting '%s'. %s: %s." % (val, setting_name, e.__class__.__name__, e)
        raise ImportError(msg)


# TODO: Raise more errors

class NodeTreeSettings(object):
    """
    A settings object, that allows NodeTree settings to be accessed as properties.
    For example:

        from nodetree.settings import nodetree_settings
        print nodetree.DEFAULT_CONTENT_SERIALIZER

    Any setting with string import paths will be automatically resolved
    and return the class, rather than the string literal.
    """
    def __init__(self, user_settings=None, defaults=None, import_strings=None):
        self.user_settings = user_settings or {}
        self.defaults = defaults or {}
        self.import_strings = import_strings or ()

    def __getattr__(self, attr):
        if attr not in self.defaults.keys():
            raise AttributeError("Invalid NodeTree setting: '%s'" % attr)

        try:
            # Check if present in user settings
            val = self.user_settings[attr]
        except KeyError:
            # Fall back to defaults
            val = self.defaults[attr]

        # Coerce import strings into classes
        if val and attr in self.import_strings:
            val = perform_import(val, attr)

        self.validate_setting(attr, val)

        # Cache the result
        setattr(self, attr, val)
        return val

    def validate_setting(self, attr, val):
        pass

nodetree_settings = NodeTreeSettings(USER_SETTINGS, DEFAULTS, IMPORT_STRINGS)


# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import inspect

from django.core.exceptions import ValidationError, ObjectDoesNotExist
from django.core.urlresolvers import get_script_prefix, resolve
from django.utils.translation import ugettext_lazy as _
from django import forms

from rest_framework.compat import  urlparse
from rest_framework import six
from rest_framework import serializers
from rest_framework.exceptions import ConfigurationError
from rest_framework.relations import RelatedField
from rest_framework.serializers import NestedValidationError

import nodetree


class GenericRelatedField(serializers.WritableField):
    """
    Represents a generic relation foreign key.
    It's actually more of a wrapper, that delegates the logic to registered serializers based on the `Model` class.
    """
    default_error_messages = {
        'no_model_match': _('Invalid model - model not available.'),
        'no_url_match': _('Invalid hyperlink - No URL match'),
        'incorrect_url_match': _('Invalid hyperlink - view name not available'),
    }

    form_field_class = forms.URLField

    def __init__(self, serializers, *args, **kwargs):
        """
        Needs an extra parameter `serializers` which has to be a dict key: value being `Model`: serializer.
        """
        super(GenericRelatedField, self).__init__(*args, **kwargs)
        self.serializers = serializers
        for model, serializer in six.iteritems(self.serializers):
            # We have to do it, because the serializer can't access a explicit manager through the
            # GenericForeignKey field on the model.
            if hasattr(serializer, 'queryset') and serializer.queryset is None:
                serializer.queryset = model._default_manager.all()

    def field_to_native(self, obj, field_name):
        """
        Delegates to the `to_native` method of the serializer registered under obj.__class__
        """
        value = super(GenericRelatedField, self).field_to_native(obj, field_name)
        serializer = self.determine_deserializer_for_data(value)

        # Necessary because of context, field resolving etc.
        serializer.initialize(self.parent, field_name)
        return serializer.to_native(value)

    def to_native(self, value):
        # Override to prevent the simplifying process of value as present in `WritableField.to_native`.
        return value

    def from_native(self, value):
        # Get the serializer responsible for input resolving
        serializer = self.determine_serializer_for_data(value)
        serializer.initialize(self.parent, self.source)
        args = [value]
        if len(inspect.getargspec(serializer.from_native).args) > 2:
            args.append(None)
        return serializer.from_native(*args)

    def determine_deserializer_for_data(self, value):
        try:
            model = value.__class__
            serializer = self.serializers[model]
        except KeyError:
            raise ValidationError(self.error_messages['no_model_match'])
        return serializer

    def determine_serializer_for_data(self, value):
        for serializer in six.itervalues(self.serializers):
            if not isinstance(serializer, serializers.HyperlinkedRelatedField):
                raise ConfigurationError('Please use HyperlinkedRelatedField as serializers on GenericRelatedField \
                instances with read_only=False or set read_only=True.')

        # This excerpt is an exact copy of ``rest_framework.relations.HyperlinkedRelatedField``, Line 363
        # From here until ...
        try:
           http_prefix = value.startswith('http:') or value.startswith('https:')
        except AttributeError:
           msg = self.error_messages['incorrect_type']
           raise ValidationError(msg % type(value).__name__)

        if http_prefix:
            # If needed convert absolute URLs to relative path
            value = urlparse.urlparse(value).path
            prefix = get_script_prefix()
            if value.startswith(prefix):
                value = '/' + value[len(prefix):]
        try:
            match = resolve(value)
        except Exception:
            raise ValidationError(self.error_messages['no_url_match'])

        # ... here

        matched_serializer = None
        for serializer in six.itervalues(self.serializers):
            if serializer.view_name == match.url_name:
                matched_serializer = serializer

        if matched_serializer is None:
            raise ValidationError(self.error_messages['incorrect_url_match'])
        return matched_serializer


class RecursiveRelatedField(RelatedField):

    def __init__(self, max_depth=-1, leaf_serializer=None, *args, **kwargs):
        super(RecursiveRelatedField, self).__init__(*args, **kwargs)
        # Forced read only.
        self.max_depth = max_depth
        self.leaf_serializer = leaf_serializer
        self.read_only = True

    def field_to_native(self, obj, field_name):

        serializer_class = self.parent.__class__
        serializer = serializer_class()
        serializer.initialize(self.parent, field_name)


        if self.max_depth > -1:
            # We decrease max_depth here
            if self.max_depth > 0:
                serializer.fields[field_name].max_depth = self.max_depth - 1
            else:
                # Here's the handling of the leaf case if no leaf serializer is given.
                if self.leaf_serializer is None:
                    return [] if self.many else None
                elif not self.many:
                    # Here's the handling of the leaf case if a leaf serializer is given and `many` is set to `False`.
                    self.leaf_serializer.initialize(self.parent, field_name)
                    return self.leaf_serializer.to_native(getattr(obj, self.source or field_name))

        if self.many:
            related_manager = getattr(obj, self.source or field_name)
            if not obj.__class__ == related_manager.model:
                raise Exception('`RecursiveRelatedField` must point at a self-referencing relation.')
            queryset = related_manager.all()
            if not self.max_depth == 0:
                return [serializer.to_native(item) for item in queryset]
            else:
                self.leaf_serializer.initialize(self.parent, field_name)
                return [self.leaf_serializer.to_native(item) for item in queryset]

        try:
            queryset = getattr(obj, self.source or field_name)
            if not obj.__class__ == queryset.__class__:
                raise Exception('`RecursiveRelatedField` must point at a self-referencing relation.')
        except ObjectDoesNotExist:
            return None
        return serializer.to_native(queryset)

    def to_native(self, value):
        # Override to prevent simplifying process as present in `WritableField`.
        return value


class ContentField(GenericRelatedField):

    nodetree_error_messages = {
        'no_node_type_match': _('Invalid node type - model not available.'),
    }

    def __init__(self, *args, **kwargs):
        super(ContentField, self).__init__(error_messages=self.nodetree_error_messages, *args, **kwargs)

    def determine_serializer_for_data(self, value):

        node_type = self.parent.init_data.get('node_type', None)
        node_type_obj = nodetree.app.get_node_type(node_type)

        if node_type_obj is None:
            raise ValidationError(self.error_messages['no_node_type_match'])
        return node_type_obj.serializer_class()

    def from_native(self, value):
        # Get the serializer responsible for input resolving
        serializer = self.determine_serializer_for_data(value)
        serializer.initialize(self.parent, self.source)
        args = [value]
        if len(inspect.getargspec(serializer.from_native).args) > 2:
            args.append(None)
        result = serializer.from_native(*args)
        if not result:
            raise NestedValidationError(serializer.errors)
        return result


from __future__ import unicode_literals
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes.generic import GenericRelation, GenericForeignKey
from django.core.exceptions import ValidationError
from django.db import models
from django.test import TestCase, RequestFactory

from rest_framework import serializers
from rest_framework.compat import patterns, url
from rest_framework.exceptions import ConfigurationError
from rest_framework.reverse import reverse

from nodetree.fields import GenericRelatedField

factory = RequestFactory()
request = factory.get('/')  # Just to ensure we have a request in the serializer context

def dummy_view(request, pk):
    pass

urlpatterns = patterns('',
    url(r'^bookmark/(?P<pk>[0-9]+)/$', dummy_view, name='bookmark-detail'),
    url(r'^note/(?P<pk>[0-9]+)/$', dummy_view, name='note-detail'),
    url(r'^tag/(?P<pk>[0-9]+)/$', dummy_view, name='tag-detail'),
    url(r'^contact/(?P<my_own_slug>[-\w]+)/$', dummy_view, name='contact-detail'),
)


class Tag(models.Model):
    """
    Tags have a descriptive slug, and are attached to an arbitrary object.
    """
    tag = models.SlugField()
    content_type = models.ForeignKey(ContentType)
    object_id = models.PositiveIntegerField()
    tagged_item = GenericForeignKey('content_type', 'object_id')

    def __unicode__(self):
        return self.tag


class Bookmark(models.Model):
    """
    A URL bookmark that may have multiple tags attached.
    """
    url = models.URLField()
    tags = GenericRelation(Tag)

    def __unicode__(self):
        return 'Bookmark: %s' % self.url


class Note(models.Model):
    """
    A textual note that may have multiple tags attached.
    """
    text = models.TextField()
    tags = GenericRelation(Tag)

    def __unicode__(self):
        return 'Note: %s' % self.text


class BookmarkSerializer(serializers.ModelSerializer):
    class Meta:
        model = Bookmark
        exclude = ('id', )


class NoteSerializer(serializers.ModelSerializer):
    class Meta:
        model = Note
        exclude = ('id', )


class TestGenericRelatedFieldDeserialization(TestCase):

    urls = 'nodetree.tests.relations_generic'

    def setUp(self):
        self.bookmark = Bookmark.objects.create(url='https://www.djangoproject.com/')
        Tag.objects.create(tagged_item=self.bookmark, tag='django')
        Tag.objects.create(tagged_item=self.bookmark, tag='python')
        self.note = Note.objects.create(text='Remember the milk')
        Tag.objects.create(tagged_item=self.note, tag='reminder')

    def test_relations_as_hyperlinks(self):

        class TagSerializer(serializers.ModelSerializer):
            tagged_item = GenericRelatedField({
                    Bookmark: serializers.HyperlinkedRelatedField(view_name='bookmark-detail'),
                    Note: serializers.HyperlinkedRelatedField(view_name='note-detail'),
                }, source='tagged_item', read_only=True)

            class Meta:
                model = Tag
                exclude = ('id', 'content_type', 'object_id', )

        serializer = TagSerializer(Tag.objects.all(), many=True, context={'request': request})
        expected = [
            {
                'tagged_item': 'http://testserver/bookmark/1/',
                'tag': 'django',
            },
            {
                'tagged_item': 'http://testserver/bookmark/1/',
                'tag': 'python',
            },
            {
                'tagged_item': 'http://testserver/note/1/',
                'tag': 'reminder'
            }
        ]
        self.assertEqual(serializer.data, expected)

    def test_relations_as_nested(self):

        class TagSerializer(serializers.ModelSerializer):
            tagged_item = GenericRelatedField({
                Bookmark: BookmarkSerializer(),
                Note: NoteSerializer(),
            }, source='tagged_item', read_only=True)

            class Meta:
                model = Tag
                exclude = ('id', 'content_type', 'object_id', )

        serializer = TagSerializer(Tag.objects.all(), many=True)
        expected = [
            {
                'tagged_item': {
                    'url': 'https://www.djangoproject.com/'
                },
                'tag': 'django'
            },
            {
                'tagged_item': {
                    'url': 'https://www.djangoproject.com/'
                },
                'tag': 'python'
            },
            {
                'tagged_item': {
                    'text': 'Remember the milk',
                },
                'tag': 'reminder'
            }
        ]
        self.assertEqual(serializer.data, expected)

    def test_mixed_serializers(self):
        class TagSerializer(serializers.ModelSerializer):
            tagged_item = GenericRelatedField({
                Bookmark: BookmarkSerializer(),
                Note: serializers.HyperlinkedRelatedField(view_name='note-detail'),
            }, source='tagged_item', read_only=True)

            class Meta:
                model = Tag
                exclude = ('id', 'content_type', 'object_id', )

        serializer = TagSerializer(Tag.objects.all(), many=True, context={'request': request})
        expected = [
            {
                'tagged_item': {
                    'url': 'https://www.djangoproject.com/'
                },
                'tag': 'django'
            },
            {
                'tagged_item': {
                    'url': 'https://www.djangoproject.com/'
                },
                'tag': 'python'
            },
            {
                'tagged_item': 'http://testserver/note/1/',
                'tag': 'reminder'
            }
        ]
        self.assertEqual(serializer.data, expected)

    def test_invalid_model(self):
        # Leaving out the Note model should result in a ValidationError
        class TagSerializer(serializers.ModelSerializer):
            tagged_item = GenericRelatedField({
                Bookmark: BookmarkSerializer(),
            }, source='tagged_item', read_only=True)

            class Meta:
                model = Tag
                exclude = ('id', 'content_type', 'object_id', )
        serializer = TagSerializer(Tag.objects.all(), many=True)

        def call_data():
            return serializer.data
        self.assertRaises(ValidationError, call_data)


class TestGenericRelatedFieldSerialization(TestCase):

    urls = 'nodetree.tests.relations_generic'

    def setUp(self):
        self.bookmark = Bookmark.objects.create(url='https://www.djangoproject.com/')
        Tag.objects.create(tagged_item=self.bookmark, tag='django')
        Tag.objects.create(tagged_item=self.bookmark, tag='python')
        self.note = Note.objects.create(text='Remember the milk')

    def test_hyperlink_serialization(self):
        class TagSerializer(serializers.ModelSerializer):
            tagged_item = GenericRelatedField({
                    Bookmark: serializers.HyperlinkedRelatedField(view_name='bookmark-detail'),
                    Note: serializers.HyperlinkedRelatedField(view_name='note-detail'),
                }, source='tagged_item', read_only=False)

            class Meta:
                model = Tag
                exclude = ('id', 'content_type', 'object_id', )

        serializer = TagSerializer(data={
            'tag': 'reminder',
            'tagged_item': reverse('note-detail', kwargs={'pk': self.note.pk})
        }, context={'request': request})
        serializer.is_valid()
        expected = {
            'tagged_item': 'http://testserver/note/1/',
            'tag': 'reminder'
        }
        self.assertEqual(serializer.data, expected)

    def test_configuration_error(self):
        class TagSerializer(serializers.ModelSerializer):
            tagged_item = GenericRelatedField({
                    Bookmark: BookmarkSerializer(),
                    Note: serializers.HyperlinkedRelatedField(view_name='note-detail'),
                }, source='tagged_item', read_only=False)

            class Meta:
                model = Tag
                exclude = ('id', 'content_type', 'object_id', )

        serializer = TagSerializer(data={
            'tag': 'reminder',
            'tagged_item': reverse('note-detail', kwargs={'pk': self.note.pk})
        }, context={'request': request})
        self.assertRaises(ConfigurationError, serializer.is_valid)

    def test_not_registered_view_name(self):
        class TagSerializer(serializers.ModelSerializer):
            tagged_item = GenericRelatedField({
                    Bookmark: serializers.HyperlinkedRelatedField(view_name='bookmark-detail'),
                }, source='tagged_item', read_only=False)

            class Meta:
                model = Tag
                exclude = ('id', 'content_type', 'object_id', )

        serializer = TagSerializer(data={
            'tag': 'reminder',
            'tagged_item': reverse('note-detail', kwargs={'pk': self.note.pk})
        })
        self.assertFalse(serializer.is_valid())

    def test_invalid_url(self):
        class TagSerializer(serializers.ModelSerializer):
            tagged_item = GenericRelatedField({
                    Bookmark: serializers.HyperlinkedRelatedField(view_name='bookmark-detail'),
                }, source='tagged_item', read_only=False)

            class Meta:
                model = Tag
                exclude = ('id', 'content_type', 'object_id', )

        serializer = TagSerializer(data={
            'tag': 'reminder',
            'tagged_item': 'foo-bar'
        })
        self.assertFalse(serializer.is_valid())

    def test_serializer_save(self):
        class TagSerializer(serializers.ModelSerializer):
            tagged_item = GenericRelatedField({
                    Bookmark: serializers.HyperlinkedRelatedField(view_name='bookmark-detail'),
                    Note: serializers.HyperlinkedRelatedField(view_name='note-detail'),
                }, source='tagged_item', read_only=False)

            class Meta:
                model = Tag
                exclude = ('id', 'content_type', 'object_id', )

        serializer = TagSerializer(data={
            'tag': 'reminder',
            'tagged_item': reverse('note-detail', kwargs={'pk': self.note.pk})
        })
        serializer.is_valid()
        expected = {
            'tagged_item': '/note/1/',
            'tag': 'reminder'
        }
        serializer.save()
        tag = Tag.objects.get(pk=3)
        self.assertEqual(tag.tagged_item, self.note)


# -*- coding: utf-8 -*- 
from __future__ import unicode_literals
import nodetree
from .models import Town, Area, House, Person

class TownNodeType(nodetree.NodeType):

    name = 'town'
    model_class = Town

    is_root = True
    allowed_children = ['area', 'house', 'person']


class AreaNodeType(nodetree.NodeType):

    name = 'area'
    model_class = Area

    allowed_children = ['house', 'person', 'area']


class HouseNodeType(nodetree.NodeType):

    name = 'house'
    model_class = House
    allowed_children = ['person']


class PersonNodeType(nodetree.NodeType):

    name = 'person'
    model_class = Person
    is_root = True
    is_leaf = True

nodetree.app.register(TownNodeType)
nodetree.app.register(AreaNodeType)
nodetree.app.register(HouseNodeType)
nodetree.app.register(PersonNodeType)


from __future__ import unicode_literals
from django.db import models

from nodetree.models import BaseContent

class Town(BaseContent):

    name = models.CharField(max_length=63)


class Area(BaseContent):

    name = models.CharField(max_length=63)


class House(BaseContent):

    name = models.CharField(max_length=63)


class Person(BaseContent):

    first_name = models.CharField(max_length=63)
    last_name = models.CharField(max_length=63)


# -*- coding: utf-8 -*- 
from __future__ import unicode_literals

from django.test import TestCase, RequestFactory
from django.utils import unittest

from rest_framework.compat import patterns, include, url

import nodetree
from nodetree.models import Node
from nodetree.viewsets import NodeViewSet
from nodetree.filters import microfilters

from .models import Town, Area, House

nodetree.autodiscover()
urlpatterns = patterns('',
    url(r'^nodetree/', include(nodetree.app.urls)),
)

factory = RequestFactory()
request = factory.get('/')


class TestNodeFilterSet(TestCase):

    urls = 'nodetree.tests.filters'

    def setUp(self):

        self.town = Town.objects.create(name='Zurich')
        self.town_node = Node.objects.create(node_type='town', content=self.town)

        self.area = Area.objects.create(name='Wiedikon')
        self.area_node = Node.objects.create(node_type='area', content=self.area, parent=self.town_node)

        self.area_2 = Area.objects.create(name='Seefeld')
        self.area_node_2 = Node.objects.create(node_type='area', content=self.area_2, parent=self.area_node)

        self.house = House.objects.create(name='Kalkbreitestrasse 98')
        self.house_node = Node.objects.create(node_type='house', content=self.house, parent=self.area_node_2)

    @unittest.skipUnless(microfilters, 'microfilters not installed')
    def test_node_type_filter(self):
        request = factory.get('/?node_type=town,area')
        view = NodeViewSet.as_view({'get': 'list'})
        response = view(request)

        expected = [
            {
                'url': 'http://testserver/nodetree/node/1/',
                'parent': None,
                'content': {
                    'url': 'http://testserver/nodetree/town/1/',
                    'node': 'http://testserver/nodetree/node/1/',
                    'name': 'Zurich'
                },
                'children': [
                    'http://testserver/nodetree/node/2/'
                ],
                'node_type': 'town',
                'level': 0

            },
            {
                'url': 'http://testserver/nodetree/node/2/',
                'parent': 'http://testserver/nodetree/node/1/',
                'content': {
                    'url': 'http://testserver/nodetree/area/1/',
                    'node': 'http://testserver/nodetree/node/2/',
                    'name': 'Wiedikon'
                },
                'children': [
                    'http://testserver/nodetree/node/3/'
                ],
                'node_type': 'area',
                'level': 1
            },
            {
                'url': 'http://testserver/nodetree/node/3/',
                'parent': 'http://testserver/nodetree/node/2/',
                'content': {
                    'url': 'http://testserver/nodetree/area/2/',
                    'node': 'http://testserver/nodetree/node/3/',
                    'name': 'Seefeld'
                },
                'children': [
                    'http://testserver/nodetree/node/4/'
                ],
                'node_type': 'area',
                'level': 2
            }
        ]
        self.assertEqual(response.data, expected)

    @unittest.skipUnless(microfilters, 'microfilters not installed')
    def test_filter_custom_methods(self):
        request = factory.get('/?node_type=area')
        view = NodeViewSet.as_view({'get': 'ancestors'})
        response = view(request, pk=4)
        expected = [
            {
                'url': 'http://testserver/nodetree/node/2/',
                'parent': 'http://testserver/nodetree/node/1/',
                'content': {
                    'url': 'http://testserver/nodetree/area/1/',
                    'node': 'http://testserver/nodetree/node/2/',
                    'name': 'Wiedikon'
                },
                'children': [
                    'http://testserver/nodetree/node/3/'
                ],
                'node_type': 'area',
                'level': 1
            },
            {
                'url': 'http://testserver/nodetree/node/3/',
                'parent': 'http://testserver/nodetree/node/2/',
                'content': {
                    'url': 'http://testserver/nodetree/area/2/',
                    'node': 'http://testserver/nodetree/node/3/',
                    'name': 'Seefeld'
                },
                'children': [
                    'http://testserver/nodetree/node/4/'
                ],
                'node_type': 'area',
                'level': 2
            }
        ]
        self.assertEqual(expected, response.data)


    @unittest.skipUnless(microfilters, 'microfilters not installed')
    def test_level_filter(self):
        request = factory.get('/?from_level=1&to_level=2')
        view = NodeViewSet.as_view({'get': 'list'})
        response = view(request, pk=4)
        expected = [
            {
                'url': 'http://testserver/nodetree/node/2/',
                'parent': 'http://testserver/nodetree/node/1/',
                'content': {
                    'url': 'http://testserver/nodetree/area/1/',
                    'node': 'http://testserver/nodetree/node/2/',
                    'name': 'Wiedikon'
                },
                'children': [
                    'http://testserver/nodetree/node/3/'
                ],
                'node_type': 'area',
                'level': 1
            },
            {
                'url': 'http://testserver/nodetree/node/3/',
                'parent': 'http://testserver/nodetree/node/2/',
                'content': {
                    'url': 'http://testserver/nodetree/area/2/',
                    'node': 'http://testserver/nodetree/node/3/',
                    'name': 'Seefeld'
                },
                'children': [
                    'http://testserver/nodetree/node/4/'
                ],
                'node_type': 'area',
                'level': 2
            }
        ]
        self.assertEqual(expected, response.data)


from __future__ import unicode_literals
from django.test import TestCase
from django.db import models

from rest_framework import serializers

from nodetree.fields import RecursiveRelatedField


class TreeModel(models.Model):

    name = models.CharField(max_length=127)
    parent = models.ForeignKey('self', null=True, related_name='children')

    def __unicode__(self):
        return self.name


class TreeSerializer(serializers.ModelSerializer):

    children = RecursiveRelatedField(many=True)

    class Meta:
        model = TreeModel
        exclude = ('id', )


class MaxDepthTreeSerializer(serializers.ModelSerializer):

    children = RecursiveRelatedField(many=True, max_depth=1)

    class Meta:
        model = TreeModel
        exclude = ('id', )


class ChainModel(models.Model):

    name = models.CharField(max_length=127)
    previous = models.OneToOneField('self', null=True, related_name='next')

    def __unicode__(self):
        return self.name


class ChainSerializer(serializers.ModelSerializer):

    next = RecursiveRelatedField(many=False)

    class Meta:
        model = ChainModel
        exclude = ('id', )


class TestRecursiveRelatedField(TestCase):

    def setUp(self):
        self.tree_root = TreeModel.objects.create(name='Tree Root')
        tree_depth_1_children = []

        for x in range(0, 3):
            tree_depth_1_children.append(TreeModel.objects.create(name='Child 1:%d' % x, parent=self.tree_root))

        for x in range(0, 2):
            TreeModel.objects.create(name='Child 2:%d' % x, parent=tree_depth_1_children[1])

        self.chain_root = ChainModel.objects.create(name='Chain Root')
        current = self.chain_root
        for x in range(0, 3):
            chain_link = ChainModel.objects.create(name='Chain link %d' % x, previous=current)
            current = chain_link


    def test_many(self):
        serializer = TreeSerializer(self.tree_root)
        expected = {
            'children': [
                {
                    'children': [],
                    'name': 'Child 1:0',
                    'parent': 1
                },
                {
                    'children': [
                        {
                            'children': [],
                            'name': 'Child 2:0',
                            'parent': 3
                        },
                        {
                            'children': [],
                            'name': 'Child 2:1',
                            'parent': 3
                        }
                    ],
                    'name': 'Child 1:1',
                    'parent': 1
                },
                {
                    'children': [],
                    'name': 'Child 1:2',
                    'parent': 1
                }
            ],
            'name': 'Tree Root',
            'parent': None
        }
        self.assertEqual(serializer.data, expected)

    def test_one(self):
        serializer = ChainSerializer(self.chain_root)
        expected = {
            'next':
                {
                    'next':
                        {'next':
                             {'next':  None,
                              'name': 'Chain link 2',
                              'previous': 3},
                         'name': 'Chain link 1',
                         'previous': 2},
                    'name': 'Chain link 0',
                    'previous': 1},
            'name': 'Chain Root',
            'previous': None
        }
        self.assertEqual(serializer.data, expected)

    def test_max_depth(self):
        serializer = MaxDepthTreeSerializer(self.tree_root)
        expected = {
            'children': [
                {
                    'children': [],
                    'name': 'Child 1:0',
                    'parent': 1
                },
                {
                    'children': [],
                    'name': 'Child 1:1',
                    'parent': 1
                },
                {
                    'children': [],
                    'name': 'Child 1:2',
                    'parent': 1
                }
            ],
            'name': 'Tree Root',
            'parent': None
        }
        self.assertEqual(serializer.data, expected)




# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.contrib.contenttypes.generic import GenericForeignKey, GenericRelation
from django.contrib.contenttypes.models import ContentType
from django.db import models

from mptt.fields import TreeForeignKey
from mptt.models import MPTTModel


class Node(MPTTModel):
    """
    The ``Node`` model is actually nothing more than an inheritor of ``MPTTModel`` with additional fields needed
    to create a generic relation and a ``node_type`` field for easily identifying the related ``NodeType``.
    Every ``BaseContent`` that resides within a tree is tied to a ``Node`` object. The ``Node`` object is only
    responsible for logic that is related to tree structure and modification.

    For more information about the inner workings of the tree implementation, please visit
    `Django MPTT's Documentation <http://django-mptt.readthedocs.org/en/latest/models.html#models-and-managers>`_.
    """

    # Generic relation fields
    content_type = models.ForeignKey(ContentType, null=True)
    object_id = models.PositiveIntegerField(null=True)
    content = GenericForeignKey()

    # MPTT parent field
    parent = TreeForeignKey('self', null=True, blank=True, related_name='children')

    # Node type field
    node_type = models.SlugField(max_length=31)

    def __unicode__(self):
        return '%s: %s %d' % ('Node', self.content, self.pk or 0)

    def delete(self, force_delete_content=True, *args, **kwargs):
        """
        This deletes the ``Node`` object. Tree adjustments are again handled by MPTT.

        :param force_delete_content: If set to ``False`` the related ``BaseContent`` object will not get deleted.
        :type force_delete_content: bool

        .. note::
            Please note that you should not set ``force_delete_content`` to ``False`` unless you have a good
            reason to keep your content objects for further use.

        """
        if force_delete_content:
            self.content.delete()
        super(Node, self).delete(*args, **kwargs)

    class Meta:
        # Make sure that it's impossible to connect two ``Node`` objects with the same ``BaseContent``.
        unique_together = (('content_type', 'object_id'),)


class BaseContent(models.Model):
    """
    As for now the abstract ``BaseContent`` model provides only basic features to establish a reverse relation
    to its ``Node`` object. It still should be used as parent class for all models you want to register with NodeTree,
    because of possible future changes to the ``BaseContent`` implementation.
    """
    nodes = GenericRelation(Node)

    @property
    def node(self):
        """
        :returns: ``nodetree.models.Node`` -- The related ``Node`` object if existing, else ``None``
        """
        if self.nodes.count() > 0:
            return self.nodes.all()[0]
        else:
            return None

    class Meta:
        abstract = True


# -*- coding: utf-8 -*- 
from __future__ import unicode_literals
import json

from django.test import TestCase, RequestFactory

from rest_framework.compat import patterns, include, url

import nodetree
from nodetree.models import Node

from .models import Town, Area, House
from nodetree.viewsets import NodeViewSet

nodetree.autodiscover()
urlpatterns = patterns('',
    url(r'^nodetree/', include(nodetree.app.urls)),
)

factory = RequestFactory()
request = factory.get('/')

class TestNodeListView(TestCase):

    urls = 'nodetree.tests.viewsets'

    def setUp(self):
        self.town_1 = Town.objects.create(name='Zurich')
        self.town_node_1 = Node.objects.create(node_type='town', content=self.town_1)

        self.town_2 = Town.objects.create(name='Bern')
        self.town_node_2 = Node.objects.create(node_type='town', content=self.town_2)

        self.town_3 = Town.objects.create(name='Lucerne')
        self.town_node_3 = Node.objects.create(node_type='town', content=self.town_3)
        self.view = NodeViewSet.as_view({'get': 'list', 'post': 'create'})

    def test_get_list(self):
        request = factory.get('/')
        response = self.view(request)
        expected = [
            {
                'url': 'http://testserver/nodetree/node/1/',
                'parent': None,
                'children': [],
                'content': {
                    'url': 'http://testserver/nodetree/town/1/',
                    'node': 'http://testserver/nodetree/node/1/',
                    'name': 'Zurich'
                },
                'node_type': 'town',
                'level': 0
            },
            {
                'url': 'http://testserver/nodetree/node/2/',
                'parent': None,
                'children': [],
                'content': {
                    'url': 'http://testserver/nodetree/town/2/',
                    'node': 'http://testserver/nodetree/node/2/',
                    'name': 'Bern'
                },
                'node_type': 'town',
                'level': 0

            },
            {
                'url': 'http://testserver/nodetree/node/3/',
                'parent': None,
                'children': [],
                'content': {
                    'url': 'http://testserver/nodetree/town/3/',
                    'node': 'http://testserver/nodetree/node/3/',
                    'name': 'Lucerne'
                },
                'node_type': 'town',
                'level': 0
            }
        ]
        self.assertEqual(expected, response.data)

    def test_invalid_post_list(self):
        request = factory.post('/', data={'node_type': 'foo'})
        response = self.view(request)
        expected = {
            'content': ['This field is required.'],
            'node_type': ["Node type 'foo' does not exist."]
        }
        self.assertEqual(expected, response.data)

    def test_post_list(self):
        request = factory.post('/', json.dumps({
                'node_type': 'area',
                'target': 'http://testserver/nodetree/node/1/',
                'position': 'first-child',
                'content': {
                    'name': 'Wiedikon',
                }
            }), content_type='application/json')
        response = self.view(request)

        expected = {
            'url': 'http://testserver/nodetree/node/4/',
            'parent': 'http://testserver/nodetree/node/1/',
            'children': [],
            'content': {
                'url': 'http://testserver/nodetree/area/1/',
                'node': 'http://testserver/nodetree/node/4/',
                'name': 'Wiedikon'
            },
            'node_type': 'area',
            'level': 1
        }
        self.assertEqual(expected, response.data)
        self.assertEqual(Area.objects.get(pk=1).name, 'Wiedikon')



class TestNodeDetailView(TestCase):

    urls = 'nodetree.tests.viewsets'

    def setUp(self):
        self.town_1 = Town.objects.create(name='Zurich')
        self.town_node_1 = Node.objects.create(node_type='town', content=self.town_1)

        self.town_2 = Town.objects.create(name='Bern')
        self.town_node_2 = Node.objects.create(node_type='town', content=self.town_2)

        self.town_3 = Town.objects.create(name='Lucerne')
        self.town_node_3 = Node.objects.create(node_type='town', content=self.town_3)

        self.area = Area.objects.create(name='Elfenau')
        self.area_node = Node.objects.create(node_type='area', content=self.area)
        self.view = NodeViewSet.as_view({'get': 'retrieve', 'put': 'update', 'delete': 'destroy'})

    def test_get_detail(self):
        request = factory.get('/')
        response = self.view(request, pk=1)
        expected = {
            'url': 'http://testserver/nodetree/node/1/',
            'parent': None,
            'children': [],
            'content': {
                'url': 'http://testserver/nodetree/town/1/',
                'node': 'http://testserver/nodetree/node/1/',
                'name': 'Zurich'
            },
            'node_type': 'town',
            'level': 0
        }
        self.assertEqual(expected, response.data)

    def test_invalid_put_detail(self):
        request = factory.put('/', json.dumps({
           'node_type': 'area',
           'target': 'http://testserver/nodetree/node/2/',
           'position': 'first-child',
        }), content_type='application/json')

        response = self.view(request, pk=1)
        expected = {
            'non_field_errors': [
                "You can't add node type 'town' as child of 'town'."
            ]
        }
        self.assertEqual(expected, response.data)

    def test_put_detail(self):
        request = factory.put('/', json.dumps({
           'target': 'http://testserver/nodetree/node/1/',
           'position': 'first-child',
        }), content_type='application/json')

        response = self.view(request, pk=4)
        expected = {
            'url': 'http://testserver/nodetree/node/4/',
            'parent': 'http://testserver/nodetree/node/1/',
            'children': [],
            'content': {
                'url': 'http://testserver/nodetree/area/1/',
                'node': 'http://testserver/nodetree/node/4/',
                'name': 'Elfenau'
            },
            'node_type': 'area',
            'level': 1
        }
        self.assertEqual(expected, response.data)

    def test_delete_detail(self):
        request = factory.delete('/')
        response = self.view(request, pk=3)

        self.assertEqual(204, response.status_code)
        self.assertEqual(Node.objects.count(), 3)


class TestCustomNodeDetailMethods(TestCase):

    urls = 'nodetree.tests.viewsets'

    def setUp(self):
        self.town = Town.objects.create(name='Zurich')
        self.town_node = Node.objects.create(node_type='town', content=self.town)

        self.area = Area.objects.create(name='Wiedikon')
        self.area_node = Node.objects.create(node_type='area', content=self.area, parent=self.town_node)

        self.house = House.objects.create(name='Kalkbreitestrasse 98')
        self.house_node = Node.objects.create(node_type='house', content=self.house, parent=self.area_node)

    def test_invalid_ancestors(self):
        request = factory.get('/')
        view = NodeViewSet.as_view({'get': 'ancestors'})
        response = view(request, pk=16)
        self.assertEqual(response.status_code, 404)

    def test_invalid_descendants(self):
        request = factory.get('/')
        view = NodeViewSet.as_view({'get': 'descendants'})
        response = view(request)
        self.assertEqual(response.status_code, 404)

    def test_invalid_children(self):
        request = factory.get('/')
        view = NodeViewSet.as_view({'get': 'children'})
        response = view(request)
        self.assertEqual(response.status_code, 404)

    def test_invalid_siblings(self):
        request = factory.get('/')
        view = NodeViewSet.as_view({'get': 'siblings'})
        response = view(request)
        self.assertEqual(response.status_code, 404)

    def test_valid_ancestors(self):
        request = factory.get('/')
        view = NodeViewSet.as_view({'get': 'ancestors'})
        response = view(request, pk=3)
        expected = [
            {
                'url': 'http://testserver/nodetree/node/1/',
                'parent': None,
                'content': {
                    'url': 'http://testserver/nodetree/town/1/',
                    'node': 'http://testserver/nodetree/node/1/',
                    'name': 'Zurich'
                },
                'children': [
                    'http://testserver/nodetree/node/2/'
                ],
                'node_type': 'town',
                'level': 0
            },
            {
                'url': 'http://testserver/nodetree/node/2/',
                'parent': 'http://testserver/nodetree/node/1/',
                'content': {
                    'url': 'http://testserver/nodetree/area/1/',
                    'node': 'http://testserver/nodetree/node/2/',
                    'name': 'Wiedikon'
                },
                'children': [
                    'http://testserver/nodetree/node/3/'
                ],
                'node_type': 'area',
                'level': 1
            }
        ]
        self.assertEqual(expected, response.data)

    def test_valid_descendants(self):
        request = factory.get('/')
        view = NodeViewSet.as_view({'get': 'descendants'})
        response = view(request, pk=1)
        expected = [
            {
                'url': 'http://testserver/nodetree/node/2/',
                'parent': 'http://testserver/nodetree/node/1/',
                'content': {
                    'url': 'http://testserver/nodetree/area/1/',
                    'node': 'http://testserver/nodetree/node/2/',
                    'name': 'Wiedikon'
                },
                'children': [
                    'http://testserver/nodetree/node/3/'
                ],
                'node_type': 'area',
                'level': 1
            },
            {
                'url': 'http://testserver/nodetree/node/3/',
                'parent': 'http://testserver/nodetree/node/2/',
                'content': {
                    'url': 'http://testserver/nodetree/house/1/',
                    'node': 'http://testserver/nodetree/node/3/',
                    'name': 'Kalkbreitestrasse 98'
                },
                'children': [],
                'node_type': 'house',
                'level': 2
            },
        ]
        self.assertEqual(expected, response.data)

    def test_valid_children(self):
        request = factory.get('/')
        view = NodeViewSet.as_view({'get': 'children'})
        response = view(request, pk=1)
        expected = [
            {
                'url': 'http://testserver/nodetree/node/2/',
                'parent': 'http://testserver/nodetree/node/1/',
                'content': {
                    'url': 'http://testserver/nodetree/area/1/',
                    'node': 'http://testserver/nodetree/node/2/',
                    'name': 'Wiedikon'
                },
                'children': [
                    'http://testserver/nodetree/node/3/'
                ],
                'node_type': 'area',
                'level': 1
            },
        ]
        self.assertEqual(expected, response.data)


    def test_valid_siblings(self):

        area = Area.objects.create(name='Seefeld')
        area_node = Node(node_type='area', content=area)
        area_node.insert_at(self.area_node, 'left', save=True)

        area_2 = Area.objects.create(name='Wollishofen')
        area_node_2 = Node(node_type='area', content=area_2)
        area_node_2.insert_at(self.area_node, 'right', save=True)

        request = factory.get('/')
        view = NodeViewSet.as_view({'get': 'siblings'})
        response = view(request, pk=self.area_node.pk)
        expected = [
            {
                'url': 'http://testserver/nodetree/node/4/',
                'parent': 'http://testserver/nodetree/node/1/',
                'content': {
                    'url': 'http://testserver/nodetree/area/2/',
                    'node': 'http://testserver/nodetree/node/4/',
                    'name': 'Seefeld'
                },
                'children': [],
                'node_type': 'area',
                'level': 1
            },
            {
                'url': 'http://testserver/nodetree/node/5/',
                'parent': 'http://testserver/nodetree/node/1/',
                'content': {
                    'url': 'http://testserver/nodetree/area/3/',
                    'node': 'http://testserver/nodetree/node/5/',
                    'name': 'Wollishofen'
                },
                'children': [],
                'node_type': 'area',
                'level': 1
            },
        ]
        self.assertEqual(expected, response.data)

    def test_valid_root(self):
        request = factory.get('/')
        view = NodeViewSet.as_view({'get': 'root'})
        response = view(request, pk=3)
        expected = {
            'url': 'http://testserver/nodetree/node/1/',
            'parent': None,
            'content': {
                'url': 'http://testserver/nodetree/town/1/',
                'node': 'http://testserver/nodetree/node/1/',
                'name': 'Zurich'
            },
            'children': [
                'http://testserver/nodetree/node/2/'
            ],
            'node_type': 'town',
            'level': 0
        }
        self.assertEqual(expected, response.data)


class TestDepthQueryParameter(TestCase):

    urls = 'nodetree.tests.viewsets'

    def setUp(self):
        self.town = Town.objects.create(name='Zurich')
        self.town_node = Node.objects.create(node_type='town', content=self.town)

        self.area = Area.objects.create(name='Wiedikon')
        self.area_node = Node.objects.create(node_type='area', content=self.area, parent=self.town_node)

        self.house = House.objects.create(name='Kalkbreitestrasse 98')
        self.house_node = Node.objects.create(node_type='house', content=self.house, parent=self.area_node)

    def test_invalid_depth_param(self):
        request = factory.get('/?depth=foobar')
        view = NodeViewSet.as_view({'get': 'retrieve'})
        response = view(request, pk=1)
        self.assertEqual(response.status_code, 400)

    def test_get_detail(self):
        request = factory.get('/?depth=1')
        view = NodeViewSet.as_view({'get': 'retrieve'})
        response = view(request, pk=1)
        expected = {
            'url': 'http://testserver/nodetree/node/1/',
            'parent': None,
            'children': [{
                 'url': 'http://testserver/nodetree/node/2/',
                 'parent': 'http://testserver/nodetree/node/1/',
                 'children': [
                     'http://testserver/nodetree/node/3/',
                 ],
                 'content': {
                     'url': 'http://testserver/nodetree/area/1/',
                     'node': 'http://testserver/nodetree/node/2/',
                     'name': 'Wiedikon'
                 },
                 'node_type': 'area',
                 'level': 1
            }],
            'content': {
                'url': 'http://testserver/nodetree/town/1/',
                'node': 'http://testserver/nodetree/node/1/',
                'name': 'Zurich'
            },
            'node_type': 'town',
            'level': 0
        }
        self.assertEqual(expected, response.data)


from __future__ import unicode_literals

from django.test import TestCase, RequestFactory

from rest_framework.compat import patterns, url, include

import nodetree
from nodetree.models import Node

from .models import Town, Person, House, Area

nodetree.autodiscover()
urlpatterns = patterns('',
    url(r'^nodetree/', include(nodetree.app.urls)),
)

factory = RequestFactory()
request = factory.get('/')  # Just to ensure we have a request in the serializer context

class TestContentSerializer(TestCase):

    urls = 'nodetree.tests.serializers'

    def setUp(self):
        self.content_obj = Town.objects.create(name='Zurich')
        self.node_obj = Node.objects.create(node_type='town', content=self.content_obj)

    def test_serialization(self):
        serializer = nodetree.app.get_node_type('town').serializer_class(self.content_obj, context={'request': request})
        expected = {
            'url': 'http://testserver/nodetree/town/1/',
            'node': 'http://testserver/nodetree/node/1/',
            'name': 'Zurich'
        }
        self.assertEqual(expected, serializer.data)


class TestBaseNodeSerializer(TestCase):

    urls = 'nodetree.tests.serializers'

    def setUp(self):
        self.content_obj = Town.objects.create(name='Zurich')
        self.node_obj = Node.objects.create(node_type='town', content=self.content_obj)

        self.node_serializer_class = nodetree.app.get_node_serializer_class(depth=-1)

    def test_serialization(self):
        serializer = self.node_serializer_class(self.node_obj, context={'request': request})
        expected = {
            'url': 'http://testserver/nodetree/node/1/',
            'content': {
                'url': 'http://testserver/nodetree/town/1/',
                'node': 'http://testserver/nodetree/node/1/',
                'name': 'Zurich',
            },
            'parent': None,
            'children': [],
            'node_type': 'town',
        }
        self.assertEqual(expected, serializer.data)


class TestBaseNodeSerializer(TestCase):

    urls = 'nodetree.tests.serializers'

    def setUp(self):
        self.content_obj = Town.objects.create(name='Zurich')
        self.node_obj = Node.objects.create(node_type='town', content=self.content_obj)
        self.node_serializer_class = nodetree.app.get_node_serializer_class(depth=-1)

    def test_serialization(self):
        serializer = self.node_serializer_class(self.node_obj, context={'request': request})
        expected = {
            'url': 'http://testserver/nodetree/node/1/',
            'content': {
                'url': 'http://testserver/nodetree/town/1/',
                'node': 'http://testserver/nodetree/node/1/',
                'name': 'Zurich',
            },
            'parent': None,
            'children': [],
            'node_type': 'town',
            'level': 0
        }
        self.assertEqual(expected, serializer.data)


class TestUpdateNodeSerializer(TestCase):

    urls = 'nodetree.tests.serializers'

    def setUp(self):
        self.town = Town.objects.create(name='Zurich')
        self.town_node = Node.objects.create(node_type='town', content=self.town)

        self.area = Town.objects.create(name='Wiedikon')
        self.area_node = Node(node_type='area', content=self.area)
        self.area_node.insert_at(self.town_node, 'first-child', True)

        self.house = House.objects.create(name='Kalkbreitestrasse 98')
        self.house_node = Node(node_type='house', content=self.house)
        self.house_node.insert_at(self.area_node, 'first-child', True)

        self.person = Person.objects.create(first_name='Lukas', last_name='Buenger')
        self.person_node = Node(node_type='person', content=self.person)
        self.person_node.insert_at(self.house_node, 'first-child', True)

        self.node_serializer_class = nodetree.app.get_node_serializer_class(depth=-1)
        self.update_node_serializer_class = nodetree.app.get_update_node_serializer_class(depth=-1)

    def test_non_existing_target(self):
        serializer = self.update_node_serializer_class(self.house_node, data={}, context={'request': request})
        serializer.is_valid()
        expected = {
            'non_field_errors': ["Node type 'house' can't be used as root."]
        }
        self.assertEqual(serializer.errors, expected)

    def test_non_existing_position(self):
        serializer = self.update_node_serializer_class(self.house_node, data={
            'target': 'http://testserver/nodetree/node/1/'
        }, context={'request': request})
        serializer.is_valid()
        expected = {
            'non_field_errors': ["You have to provide a 'position' in combination with a 'target'."]
        }
        self.assertEqual(serializer.errors, expected)

    def test_invalid_target(self):
        serializer = self.update_node_serializer_class(self.house_node, data={
            'target': 'http://testserver/nodetree/node/3/',
            'position': 'first-child',
        }, context={'request': request})
        serializer.is_valid()
        expected = {
            'non_field_errors': ["You can't use the object you want to move as target."]
        }
        self.assertEqual(serializer.errors, expected)

    def test_invalid_parent_due_to_position_interpretation(self):
        serializer = self.update_node_serializer_class(self.house_node, data={
            'target': 'http://testserver/nodetree/node/1/',
            'position': 'left',
        }, context={'request': request})
        serializer.is_valid()
        expected = {
            'non_field_errors': ["Node type 'house' can't be used as root."]
        }
        self.assertEqual(serializer.errors, expected)

    def test_invalid_parent(self):
        serializer = self.update_node_serializer_class(self.area_node, data={
            'target': 'http://testserver/nodetree/node/2/',
            'position': 'first-child',
        }, context={'request': request})
        serializer.is_valid()
        expected = {
            'non_field_errors': ["You can't use the object you want to move as target."]
        }
        self.assertEqual(serializer.errors, expected)

    def test_valid_parent(self):
        serializer = self.update_node_serializer_class(self.house_node, data={
            'target': 'http://testserver/nodetree/node/2/',
            'position': 'left',
        }, context={'request': request})
        serializer.is_valid()
        serializer.save()
        expected = {
            'url': 'http://testserver/nodetree/node/3/',
            'parent': 'http://testserver/nodetree/node/1/',
            'children': [
                {
                    'url': 'http://testserver/nodetree/node/4/',
                    'parent': 'http://testserver/nodetree/node/3/',
                    'children': [],
                    'content': {
                        'url': 'http://testserver/nodetree/person/1/',
                        'node': 'http://testserver/nodetree/node/4/',
                        'first_name': 'Lukas',
                        'last_name': 'Buenger'
                    },
                    'node_type': 'person',
                    'level': 2
                }
            ],
            'content':
                {
                    'url': 'http://testserver/nodetree/house/1/',
                    'node': 'http://testserver/nodetree/node/3/',
                    'name': 'Kalkbreitestrasse 98'
                },
            'node_type': 'house',
            'level': 1
        }
        self.assertEqual(expected, serializer.data)


class TestCreateNodeSerializer(TestCase):

    urls = 'nodetree.tests.serializers'

    def setUp(self):
        self.town = Town.objects.create(name='Zurich')
        self.town_node = Node.objects.create(node_type='town', content=self.town)

        self.area = Area.objects.create(name='Wiedikon')
        self.area_node = Node(node_type='area', content=self.area)
        self.area_node.insert_at(self.town_node, 'first-child', True)

        self.house = House.objects.create(name='Kalkbreitestrasse 98')
        self.house_node = Node(node_type='house', content=self.house)
        self.house_node.insert_at(self.area_node, 'first-child', True)

        self.person = Person.objects.create(first_name='Lukas', last_name='Buenger')
        self.person_node = Node(node_type='person', content=self.person)
        self.person_node.insert_at(self.house_node, 'first-child', True)
        self.node_serializer_class = nodetree.app.get_node_serializer_class(depth=-1)

        self.create_node_serializer_class = nodetree.app.get_create_node_serializer_class(depth=-1)

    def test_non_existing_node_type(self):
        serializer = self.create_node_serializer_class(data={}, context={'request': request})
        serializer.is_valid()

        expected = {'node_type': ['This field is required.'], 'content': ['This field is required.']}
        self.assertEqual(expected, serializer.errors)

    def test_invalid_node_type(self):
        serializer = self.create_node_serializer_class(data={'node_type': 'foo'}, context={'request': request})
        serializer.is_valid()

        expected = {'node_type': ["Node type 'foo' does not exist."], 'content': ['This field is required.']}
        self.assertEqual(expected, serializer.errors)

    def test_non_existing_content(self):
        serializer = self.create_node_serializer_class(data={'node_type': 'person'}, context={'request': request})
        serializer.is_valid()

        expected = {'content': ['This field is required.']}
        self.assertEqual(expected, serializer.errors)

    def test_invalid_content(self):
        serializer = self.create_node_serializer_class (data={
            'node_type': 'person',
            'content': {
                'first_name': 'Lukas',
                'last_name': '',
            }
        }, context={'request': request})
        serializer.is_valid()
        expected = {'content': [
                {'last_name': ['This field is required.']}
            ]
        }
        self.assertEqual(expected, serializer.errors)

    def test_valid_content(self):
        serializer = self.create_node_serializer_class (data={
            'node_type': 'person',
            'content': {
                'first_name': 'Lukas',
                'last_name': 'Buenger',
            }
        }, context={'request': request})
        serializer.is_valid()
        serializer.save()
        expected = {
            'url': 'http://testserver/nodetree/node/5/',
            'parent': None,
            'children': [],
            'node_type': 'person',
            'content': {
                'url': 'http://testserver/nodetree/person/2/',
                'node': 'http://testserver/nodetree/node/5/',
                'first_name': 'Lukas',
                'last_name': 'Buenger',
            },
            'level': 0
        }
        self.assertEqual(expected, serializer.data)


class TestDepthParameter(TestCase):

    urls = 'nodetree.tests.serializers'

    def setUp(self):
        self.town = Town.objects.create(name='Zurich')
        self.town_node = Node.objects.create(node_type='town', content=self.town)

        self.area = Area.objects.create(name='Wiedikon')
        self.area_node = Node(node_type='area', content=self.area)
        self.area_node.insert_at(self.town_node, 'first-child', True)

        self.house = House.objects.create(name='Kalkbreitestrasse 98')
        self.house_node = Node(node_type='house', content=self.house)
        self.house_node.insert_at(self.area_node, 'first-child', True)

        self.person = Person.objects.create(first_name='Lukas', last_name='Buenger')
        self.person_node = Node(node_type='person', content=self.person)
        self.person_node.insert_at(self.house_node, 'first-child', True)
        self.node_serializer_class = nodetree.app.get_node_serializer_class(depth=1)

    def test_max_depth(self):
        serializer = self.node_serializer_class(self.town_node, context={'request': request})
        expected = {
            'url': 'http://testserver/nodetree/node/1/',
            'parent': None, 
            'content': {
                'url': 'http://testserver/nodetree/town/1/',
                'node': 'http://testserver/nodetree/node/1/',
                'name': 'Zurich'
            }, 
            'children': [{
                    'url': 'http://testserver/nodetree/node/2/',
                    'parent': 'http://testserver/nodetree/node/1/',
                    'content': {
                        'url': 'http://testserver/nodetree/area/1/',
                        'node': 'http://testserver/nodetree/node/2/',
                        'name': 'Wiedikon'
                    }, 
                    'children': [
                        'http://testserver/nodetree/node/3/'
                    ], 
                    'node_type': 'area',
                    'level': 1
            }], 
            'node_type': 'town',
            'level': 0
        }
        self.assertEqual(serializer.data, expected)